/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "dcmotor10.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define DCMOTOR10_DUMMY 0

// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

uint16_t dev_mirror_bits ( uint16_t wr_data );

void dcmotor10_generic_transfer ( dcmotor10_t *ctx, 
                                  spi_master_transfer_data_t *block );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void dcmotor10_cfg_setup ( dcmotor10_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->inh = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_1;
}

DCMOTOR10_RETVAL dcmotor10_init ( dcmotor10_t *ctx, dcmotor10_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return DCMOTOR10_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, DCMOTOR10_DUMMY );

    // Output pins 

    digital_out_init( &ctx->inh, cfg->inh );

    return DCMOTOR10_OK;

}

void dcmotor10_send_cmd ( dcmotor10_t *ctx, uint16_t wr_data )
{
    spi_master_transfer_data_t block;
    uint16_t temp_val;
    uint8_t tx_buf[ 2 ];

    temp_val = dev_mirror_bits( wr_data );

    tx_buf[ 1 ] = ( uint8_t )( temp_val & DCMOTOR10_8_BIT_DATA );
    tx_buf[ 0 ] = ( uint8_t )( ( temp_val >> 8 ) & DCMOTOR10_8_BIT_DATA );

    block.tx_length = 2;
    block.tx_buffer = tx_buf;
    block.rx_length = 0;

    dcmotor10_generic_transfer( ctx, &block );
}

uint16_t dcmotor10_read_status ( dcmotor10_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t rx_buf[ 2 ];
    uint16_t temp_val;
    uint16_t result;

    block.tx_length = 0;
    block.rx_buffer = rx_buf;
    block.rx_length = 2;

    dcmotor10_generic_transfer( ctx, &block );

    temp_val = rx_buf[ 0 ];
    temp_val <<= 8;
    temp_val |= rx_buf[ 1 ];

    result = dev_mirror_bits( temp_val );

    return result;
}

void dcmotor10_inhibit ( dcmotor10_t *ctx, uint8_t state )
{
	if( state == DCMOTOR10_INHIBIT )
	{
        digital_out_low( &ctx->inh );
	}
	else
	{
        digital_out_high( &ctx->inh );	
	}
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

uint16_t dev_mirror_bits ( uint16_t wr_data )
{
    uint8_t count = 15;
    uint16_t tmp = wr_data;
    wr_data >>= 1;
    while ( wr_data )
    {
        tmp <<= 1;
        tmp |= wr_data & 1;
        wr_data >>= 1;
        count--;
    }
    tmp <<= count;

    return tmp;
}

void dcmotor10_generic_transfer ( dcmotor10_t *ctx, 
                                  spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

// ------------------------------------------------------------------------- END
